@@ -63,6 +63,11 @@ type Breakpoint struct {
 	// ReturnInfo describes how to collect return variables when this
 	// breakpoint is hit as a return breakpoint.
 	returnInfo *returnBreakpointInfo
+
+	// RootFuncName is the name of the root function from where tracing needs to be done
+	RootFuncName string
+	// TraceFollowCalls indicates the depth of tracing
+	TraceFollowCalls int
 }
 
 // Breaklet represents one of multiple breakpoints that can overlap on a
@@ -135,7 +140,16 @@ const (
 	// been loaded and we should try to enable suspended breakpoints.
 	PluginOpenBreakpoint
 
-	steppingMask = NextBreakpoint | NextDeferBreakpoint | StepBreakpoint
+	// StepIntoNewProc is a breakpoint used to step into a newly created
+	// goroutine.
+	StepIntoNewProcBreakpoint
+
+	// NextInactivatedBreakpoint a NextBreakpoint that has been inactivated, see rangeFrameInactivateNextBreakpoints
+	NextInactivatedBreakpoint
+
+	StepIntoRangeOverFuncBodyBreakpoint
+
+	steppingMask = NextBreakpoint | NextDeferBreakpoint | StepBreakpoint | StepIntoNewProcBreakpoint | NextInactivatedBreakpoint | StepIntoRangeOverFuncBodyBreakpoint
 )
 
 // WatchType is the watchpoint type
@@ -210,6 +224,12 @@ func (bp *Breakpoint) VerboseDescr() []string {
 			r = append(r, fmt.Sprintf("StackResizeBreakpoint Cond=%q", exprToString(breaklet.Cond)))
 		case PluginOpenBreakpoint:
 			r = append(r, "PluginOpenBreakpoint")
+		case StepIntoNewProcBreakpoint:
+			r = append(r, "StepIntoNewProcBreakpoint")
+		case NextInactivatedBreakpoint:
+			r = append(r, "NextInactivatedBreakpoint")
+		case StepIntoRangeOverFuncBodyBreakpoint:
+			r = append(r, "StepIntoRangeOverFuncBodyBreakpoint Cond=%q", exprToString(breaklet.Cond))
 		default:
 			r = append(r, fmt.Sprintf("Unknown %d", breaklet.Kind))
 		}
@@ -304,9 +324,12 @@ func (bpstate *BreakpointState) checkCond(tgt *Target, breaklet *Breaklet, threa
 			}
 		}
 
-	case StackResizeBreakpoint, PluginOpenBreakpoint:
+	case StackResizeBreakpoint, PluginOpenBreakpoint, StepIntoNewProcBreakpoint, StepIntoRangeOverFuncBodyBreakpoint:
 		// no further checks
 
+	case NextInactivatedBreakpoint:
+		active = false
+
 	default:
 		bpstate.CondError = fmt.Errorf("internal error unknown breakpoint kind %v", breaklet.Kind)
 	}
@@ -329,6 +352,9 @@ func (bpstate *BreakpointState) checkCond(tgt *Target, breaklet *Breaklet, threa
 		case StepBreakpoint:
 			bpstate.Stepping = true
 			bpstate.SteppingInto = true
+		case StepIntoRangeOverFuncBodyBreakpoint:
+			bpstate.Stepping = true
+			bpstate.SteppingIntoRangeOverFuncBody = true
 		}
 	}
 }
@@ -704,7 +730,7 @@ func (t *Target) setBreakpointInternal(logicalID int, addr uint64, kind Breakpoi
 		return bp, nil
 	}
 
-	f, l, fn := t.BinInfo().PCToLine(uint64(addr))
+	f, l, fn := t.BinInfo().PCToLine(addr)
 
 	fnName := ""
 	if fn != nil {
@@ -823,6 +849,29 @@ func (t *Target) ClearSteppingBreakpoints() error {
 	return nil
 }
 
+func (t *Target) clearInactivatedSteppingBreakpoint() error {
+	threads := t.ThreadList()
+	for _, bp := range t.Breakpoints().M {
+		for i := range bp.Breaklets {
+			if bp.Breaklets[i].Kind == NextInactivatedBreakpoint {
+				bp.Breaklets[i] = nil
+			}
+		}
+		cleared, err := t.finishClearBreakpoint(bp)
+		if err != nil {
+			return err
+		}
+		if cleared {
+			for _, thread := range threads {
+				if thread.Breakpoint().Breakpoint == bp {
+					thread.Breakpoint().Clear()
+				}
+			}
+		}
+	}
+	return nil
+}
+
 // finishClearBreakpoint clears nil breaklets from the breaklet list of bp
 // and if it is empty erases the breakpoint.
 // Returns true if the breakpoint was deleted
@@ -879,7 +928,8 @@ type BreakpointState struct {
 	Stepping bool
 	// SteppingInto is true if one of the active stepping breaklets has Kind ==
 	// StepBreakpoint.
-	SteppingInto bool
+	SteppingInto                  bool
+	SteppingIntoRangeOverFuncBody bool
 	// CondError contains any error encountered while evaluating the
 	// breakpoint's condition.
 	CondError error
@@ -949,7 +999,7 @@ func (rbpi *returnBreakpointInfo) Collect(t *Target, thread Thread) []*Variable
 		return returnInfoError("could not read function entry", err, thread.ProcessMemory())
 	}
 
-	vars, err := scope.Locals(0)
+	vars, err := scope.Locals(0, "")
 	if err != nil {
 		return returnInfoError("could not evaluate return variables", err, thread.ProcessMemory())
 	}
@@ -1010,6 +1060,10 @@ type LogicalBreakpoint struct {
 	Cond ast.Expr
 
 	UserData interface{} // Any additional information about the breakpoint
+	// Name of root function from where tracing needs to be done
+	RootFuncName string
+	// depth of tracing
+	TraceFollowCalls int
 }
 
 // SetBreakpoint describes how a breakpoint should be set.
